#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['seedweed']

package_data = \
{'': ['*']}

install_requires = \
['ecdsa >= 0.16.0']

entry_points = \
{'console_scripts': ['generate-seedweed-test-vectors = '
                     'seedweed.vectors:generate']}

setup(name='seedweed',
      version='1.0rc3',
      description='Seeded WebAuthn Credentials: reference implementation',
      author='Nicolas Stalder',
      author_email='nicolas@solokeys.com',
      url='https://github.com/solokeys/seedweed',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='~=3.6',
     )
